#! /usr/bin/octave -qf
##
## The first line of this file should point to the octave executable. 
## If you have octave binary in another directory than /usr/bin, 
## modify, please, the path. If the "/usr/bin/octave" does not work,
## you may want to try "/usr/local/bin/octave".
##
## ***************************************************************************
## *                                                                         *
## *   This program is free software; you can redistribute it and/or modify  *
## *   it under the terms of the GNU General Public License as published by  *
## *   the Free Software Foundation; either version 2 of the License, or     *
## *   (at your option) any later version.                                   *
## *                                                                         *
## ***************************************************************************
##
## Author: Jan Valenta <valenta@irsm.cas.cz>
## Created 10. 2. 2005
##
##____________________________________________________________________________
##
## Program na reseni lin rovnic pomoci singularniho rozkladu. Vstupem
## je matice soustavy (MatA) v ASCII souboru a vektor pravych stran v
## ASCII souboru
##
## pouziti: ./ComputeSVD.m soubor_matice_A soubor_vectoru_B regularizacni_clen
##
## Regularizacni clen se pricte k prvkum na hlavni diagonale a zajisti
## lepsi podminenost matice soustavy. Pokud se regularizacni clen = 0,
## tak se nic nemeni. Hodnotu clenu je potreba vyzkouset pro kazdou ulohu
## individualne. V soucasne verzi doporucuji regularizacni clen nepouzivat,
## tedy ponechat ho rovem 0.
##
##
##--------------------------------------------------------------------------


#############################################################################
## 
## User adjustable parameters follow
##
				
vel_prior = 2;		# default value of velocities (a priori information)
vel_uncert = 0.1;	# standard deviation of initial value of velocities
depth_prior = 2;	# default (initial) value of depths (a priori
			# information)
depth_uncert = 1;	# standard deviation of initial value of depths
data_uncert = 0.1;	# standard deviation of travel time errors
min_vel = 0.5;		# minimal velocity allowed in model
max_vel = 6;		# maximal velocity allowed in model
min_depth = 0.2;	# minimal depth allowed in model
max_depth = 5;		# maximal depth allowed in model
tol=0.001;		# the smallest singular value allowed in SVD

##
## End of user adjustable parameters					
##
###############################################################################

##############################################################################
## Following variables are currently not used
##
##output_precision = 6; #urcuje pocet desetinnych mist na vystupu
##halt_diff = 0.03; #pokud bude pomer hodnot vsech promennych ve nasledujicich
				#dvou iteracich mensi ne halt_diff,
				#vypocet se zastavi
#maxchyba = 0.1;	#not used			
			
#num_zero = 1e-12; #urcuje absolutni hodnotu, od niz se budou vypoctene
				#hodnoty povazovat za nulove (kvuli
				#eliminaci zaokrouhlovacich chyb) --
				#nutno hlavne pro vypocet pomoci
				#singularniho rozkladu
				#nyni se misto num_zero pouziva promenna tol
##
## End of unused variables
#############################################################################

## od verze octave 2.9.6 je argv "cell array"

#ignore_function_time_stamp="all";

argumenty = argv();

resize_on_range_error = 0; #upozorni na chybu preteceni indexu

##
## Cteni matice soustavy, MatA
##
#  [fr, msg] = fopen(argv(1), "rt", "native");
  [fr, msg] = fopen(argumenty{1}, "rt", "native");
  if fr == -1
    disp(msg)
    exit
  endif
  
  [tmp, count] = fscanf(fr, "%d%d%d%d", 4);
  XSize = tmp(1,1);
  YSize = tmp(2,1);
  ZacCtver = tmp(3,1);
  KonecProf = tmp(4,1);
  MatA = fscanf(fr, "%f", [XSize, YSize]);
#  MatA = zeros(YSize, XSize);
#  for i = 1:YSize
#    tmp = fscanf(fr, "%f", XSize);
#    for j = 1:XSize
#      MatA(i,j) = tmp(j);
#    endfor  
#    MatA(i,:) = tmp';
#  endfor
  MatA = MatA';
  
  if fclose(fr) == -1
    disp("Error, can not close file");
    disp(fopen(fr));
  endif
  clear tmp;

##
## Cteni vektoru pravych stran, VectB
##
#  [fr, msg] = fopen(argv(2), "rt", "native");
  [fr, msg] = fopen(argumenty{2}, "rt", "native");
  if fr == -1
    disp(msg)
    exit
  endif

  [tmp, count] = fscanf(fr, "%d", 1);
  BSize = tmp(1);
  VectB = fscanf(fr, "%f", [BSize, 1]);
#  VectB = zeros(BSize,1);
#  for i = 1:BSize
#    VectB(i, 1) = fscanf(fr, "%f", 1);
#  endfor
  
  
  if fclose(fr) == -1
    disp("Error, can not close file");
    disp(fopen(fr));
  endif
  clear tmp;
  

  ########################################
  ## Odstraneni nulovych sloupcu z MatA ##
  ########################################

#  nuly = any(MatA);
#  j = 0;
#  for i = 1:columns(nuly)
#	if nuly(i) == 1
#	  j = j + 1;
#	  tmpA(:,j) = MatA(:,i);
#	endif
#  endfor  
#  MatAA = MatA;
#  MatA = tmpA;
  
  
#  A = MatA' * MatA;
#  b = MatA' * VectB;
  A = MatA;
  b = VectB;
  clear MatA VectB;

  reg_clen = str2num(argumenty{3});
#  if reg_clen  != 0
#    A = MatA' * MatA;
#    b = MatA' * VectB;    
#    for i = 1:rows(A)
#      A(i,i) = A(i,i) + reg_clen;
#    endfor
#  endif
  
  # pokud existuje vysledek z prechozi iterace (soubor VectXX.csv)
  # nacteme ho, jinak se berou defaultni hodnoty
  [fr, msg] = fopen("VectXX.csv", "rt", "native");
  if fr == -1
    #pokud neexistuje VectXX.csv vytvorime vektor vstupnich rychlosti
    mpr = ones(columns(A),1) * (1/vel_prior);
	for i = 1:(ZacCtver + 1)
	  mpr(i) = depth_prior;
	endfor
  else
    #jinak nacteme vysledky predchozi iterace jako a priorni informaci
    for i = 1:(columns(A))
      tmp_pr = fscanf(fr, "%f", 1);
	  if i <= (ZacCtver + 1)
	    if (tmp_pr <= max_depth) && (tmp_pr >= min_depth)
		  mpr(i, 1) = tmp_pr;
		else
		  mpr(i, 1) = depth_prior;
		endif
	  else
	    if (tmp_pr <= (1/max_vel)) && (tmp_pr >= (1/min_vel))
		  mpr(i, 1) = tmp_pr;
		else
		  mpr(i, 1) = 1/vel_prior;
		endif      
	  endif
#	  mpr(i,1) = fscanf(fr, "%f", 1);  
    endfor 
    if fclose(fr) == -1
      disp("Error, can not close file");
      disp(fopen(fr));
    endif
  endif
  
 # #vytvorime vektor vstupnich rychlosti
 # mpr = ones(columns(A),1) * (1 / vel_prior);
  
  #vytvorime kovariancni matici nejistot vstupnich rychlosti
  Cm = zeros(columns(A));
  for i = 1:(ZacCtver + 1)
    Cm(i,i) = depth_uncert^2;
#    Cm(i,i) = 1/(depth_uncert^2);
  endfor
  for i = (ZacCtver + 1):(columns(A))
#    Cm(i,i) = 1/(vel_uncert^2);
    Cm(i,i) = (vel_uncert)^2;	
  endfor

  #vytvorime kovarianci matici nejistot merenych dat
  Cd = zeros(rows(A));
  for i = 1:(rows(A))
#    Cd(i,i) = (data_uncert)^2;
	if (b(i) > 9 && b(i) <10.2)
	  Cd(i,i) = (data_uncert*2)^2;
	else
	  Cd(i,i) = (data_uncert)^2;
	endif
  endfor
  
#  vect_X = mpr + inv(A' * inv(Cd) * A + inv(Cm)) * A' * inv(Cd) * (b - A * mpr);
#  cov_post = inv(A' * inv(Cd) * A + inv(Cm));
  Cdi = pinv(Cd);
  Cmi = pinv(Cm);
#  tmp = (A' * Cdi * A + Cmi);
#  reg_clen = 10;
#  r = 1;
#  d = 0;
#  while ((r >= 1e-6) && (reg_clen >= 1e-12))
#	d_old = tmp;
#    for i = 1:rows(tmp)
#      tmp(i,i) = tmp(i,i) + reg_clen;
#    endfor 
#	d_old = d;
#	reg_clen_old = reg_clen;
#	[d, r] = inv(tmp);
#	reg_clen = reg_clen/10;
#  endwhile
#  vnitrni_clen = d_old;	   
#  tmp_a = A' * Cdi * A + Cmi;
#  tmp_y = A' * Cdi * (b - A * mpr);
#  tmp_x = tmp_a \ tmp_y;
  vnitrni_clen = pinv(A' * Cdi * A + Cmi, tol);
  vect_X = mpr + vnitrni_clen * A' * Cdi * (b - A * mpr);
#  vect_X = mpr + tmp_x;
  cov_post = vnitrni_clen;

#plot(vect_X)
#hold on



  ############################################
  ## Nasleduje iteracni zpresnovani vypoctu ##
  ############################################
  
  ## Rektorys, II. dil, str. 562
  
#  MatAi = refract3d_invert(A);
  
#  for i=1:3
#    r = b - A * vect_X;
#    y = MatA \ r;
#    y = MatAi * r;
#    vect_X = vect_X + y;
#  endfor
#MatAi = refract3d_invert(A)
#for i=1:1
#  dx = vect_X - MatAi*b;
#  vect_X = vect_X + dx;
#  plot(vect_X)  
#endfor
  


  std_dev = zeros(columns(cov_post));  
  for i = 1:columns(cov_post)
    std_dev(i) = sqrt(cov_post(i,i));
  endfor

  vect_XX = vect_X;
  [fw, msg] = fopen("VectXX.csv", "wt", "native");
  if fw == -1
    disp(msg)
    exit
  endif

  radkyX = rows(vect_XX);  
  for i = 1:radkyX
#    fprintf(fw, "%1.8f", vect_X(radkyX - (i - 1)));
    fprintf(fw, "%1.8f", vect_XX(i));
    fprintf(fw, "\n");
  endfor

  if fclose(fw) == -1
    disp("Error, can not close file");
    disp(fopen(fw));
  endif  
  
  ##########################################
  ## Doplneni vyrazenych sloupcu do VectX ##
  ##########################################

#  tmpX = NA;
#  vect_XX = vect_X;
#  j = 0;
#  for i = 1:columns(nuly)
#    if nuly(i) == 1
#	  j = j + 1;
#	  tmpX(i,1) = vect_X(j);
#	endif
#	if nuly(i) == 0 
#	  tmpX(i,1) = -9999;
#	endif
#  endfor
#  vect_X = tmpX;

#  tmp_std = NA;
#  j = 0;
#  for i = 1:columns(nuly)
#    if nuly(i) == 1
#      j = j + 1;
#	  tmp_std(i,1) = std_dev(j);
#	endif
#	if nuly(i) == 0 
#	  tmp_std(i,1) = 9999;
#	endif
#  endfor
#  vect_std = tmp_std;
   vect_std = std_dev;

  
  [fw, msg] = fopen("VectX.csv", "wt", "native");
  if fw == -1
    disp(msg)
    exit
  endif

  [fw_std, msg] = fopen("VectX_std.csv", "wt", "native");
  if fw == -1
    disp(msg)
    exit
  endif

  
  radkyX = rows(vect_X);
  for i = 1:radkyX
#    fprintf(fw, "%1.8f", vect_X(radkyX - (i - 1)));
    fprintf(fw, "%1.8f", vect_X(i));
    fprintf(fw, "\n");
    fprintf(fw_std, "%1.8f%s%1.8f", vect_X(i)," ",vect_std(i));
    fprintf(fw_std, "\n");	
  endfor

  if fclose(fw) == -1
    disp("Error, can not close file");
    disp(fopen(fw));
  endif  
  
  if fclose(fw_std) == -1
    disp("Error, can not close file");
    disp(fopen(fw));
  endif  

  b1 = A * vect_X;
  diff_b = b1 - b;
  for i = 1:rows(diff_b)
    diff_b(i) = diff_b(i)^2;
  endfor
  rms = sqrt(sum(diff_b)/rows(diff_b));

  [fw_rms, msg] = fopen("rms.csv", "wt", "native");
  if fw == -1
    disp(msg)
    exit
  endif  

  fprintf(fw_rms, "%.12f", rms);
  
  if fclose(fw_rms) == -1
    disp("Error, can not close file");
    disp(fopen(fw));
  endif

# save -binary "pokus.oct"
      
  clear fr;
  clear fw;
  clear fw_std;
  clear fw_rms;


exit
